/*******************************************************************************
* File Name: FIFOIn_1.c  
* Version 1.0
*
* Description:
* This component provides a convenient interface from programmable hardware to
* the CPU or DMA.  That interface is implemented using the 4-entry FIFO in the
* UDB.
*
********************************************************************************
* Copyright (2010), Cypress Semiconductor Corporation.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is 
* protected by and subject to worldwide patent protection (United States and 
* foreign), United States copyright laws and international treaty provisions. 
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable 
* license to copy, use, modify, create derivative works of, and compile the 
* Cypress Source Code and derivative works for the sole purpose of creating 
* custom software in support of licensee product to be used only in conjunction 
* with a Cypress integrated circuit as specified in the applicable agreement. 
* Any reproduction, modification, translation, compilation, or representation of 
* this software except as specified above is prohibited without the express 
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. Use may be 
* limited by and subject to the applicable Cypress software license agreement. 
*******************************************************************************/

#include "cytypes.h"
#include "FIFOIn_1.h"

uint8 FIFOIn_1_initVar = 0u;

/*******************************************************************************
* Function Name: FIFOIn_1_Enable
********************************************************************************
*
* Summary:
*  Enables FIFO capture.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void FIFOIn_1_Enable(void) 
{  
    FIFOIn_1_CONTROL_REG |= FIFOIn_1_EN;    
}

/*******************************************************************************
* Function Name: FIFOIn_1_Init
********************************************************************************
*
* Summary:
*  Initializes the component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void FIFOIn_1_Init(void) 
{
    uint8 enableInterrupts;
    
	/* Set the FIFO fill level threshold value */
	/* Only the LSB Bus Status is used, so only that Aux register needs setting */
	/* Must be done in a critical region since Aux Control is shared */
    /* Enter critical section */
    enableInterrupts = CyEnterCriticalSection();
    FIFOIn_1_LSB_AUX_CONTROL_REG &= ~FIFOIn_1_FIFO_LVL;
    FIFOIn_1_LSB_AUX_CONTROL_REG |= FIFOIn_1_LVL_SETTING;
    /* Exit critical section */
    CyExitCriticalSection(enableInterrupts);    
}

/*******************************************************************************
* Function Name: FIFOIn_1_Start
********************************************************************************
* Summary:
*  Initializes the FIFO setting for the component and enables the FIFO.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void FIFOIn_1_Start(void) 
{
    if (FIFOIn_1_initVar == 0u)
    {
        FIFOIn_1_Init();
        FIFOIn_1_initVar = 1u;
    }
      
    FIFOIn_1_Enable();
}

/*******************************************************************************
* Function Name: FIFOIn_1_Stop
********************************************************************************
* Summary:
*  Disables the component and clears the FIFO.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void FIFOIn_1_Stop(void) 
{
    FIFOIn_1_CONTROL_REG &= ~FIFOIn_1_EN;    
    FIFOIn_1_ClearFIFO();
}

/*******************************************************************************
* Function Name: FIFOIn_1_Disable
********************************************************************************
* Summary:
*  Disables the component.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void FIFOIn_1_Disable(void) 
{
    FIFOIn_1_CONTROL_REG &= ~FIFOIn_1_EN;    
}

/*******************************************************************************
* Function Name: FIFOIn_1_Read
********************************************************************************
*
* Summary:
*  Returns a single byte/word from the FIFO.
*
* Parameters:
*  void
*
* Return:
*  Byte/word containing the data received.
* 
*******************************************************************************/
#if (FIFOIn_1_BUS_WIDTH == 8u)
	uint8 FIFOIn_1_Read(void) 
	{               
		return(FIFOIn_1_FIFO_REG);
	}
#else
	uint16 FIFOIn_1_Read(void) 
	{               
		return(CY_GET_REG16(FIFOIn_1_FIFO_PTR));
	}
#endif

/*******************************************************************************
* Function Name: FIFOIn_1_ReadStatus
********************************************************************************
*
* Summary:
*  Returns state in the status register.
*
* Parameters:
*  None.
*
* Return:
*  State of the status register
*   FIFOIn_1_FIFO_OVERFLOW (Clear on Read)     
*   FIFOIn_1_FIFO_NOT_EMPTY   
*
* Side Effects: 
*  Clears the bits of status register that are Clear on Read.
*
*******************************************************************************/
uint8 FIFOIn_1_ReadStatus(void) 
{
    return (FIFOIn_1_STATUS_REG & FIFOIn_1_ST_MASK);
}
        
/*******************************************************************************
* Function Name: FIFOIn_1_SetInterruptMode
********************************************************************************
*
* Summary:
*  Sets the interrupt source.
*
* Parameters:
*  Byte containing the constant for the selected interrupt sources.
*   FIFOIn_1_FIFO_OVERFLOW     
*   FIFOIn_1_FIFO_NOT_EMPTY   
*
* Return:
*  None.
*
*******************************************************************************/
void FIFOIn_1_SetInterruptMode(uint8 interruptSource) 
{
    FIFOIn_1_STATUS_MASK_REG = (interruptSource & FIFOIn_1_ST_MASK); 
}

/*******************************************************************************
* Function Name: FIFOIn_1_ClearFIFO
********************************************************************************
*
* Summary:
*  Clears out the FIFO.  Any data present in the FIFO will be lost.
*  This call should be made only when the component is disabled.
*
* Parameters:
*  None.
*
* Return:
*  None.
* 
*******************************************************************************/
void FIFOIn_1_ClearFIFO(void) 
{
    uint8 enableInterrupts;

    /* Enter critical section */
    enableInterrupts = CyEnterCriticalSection();
    FIFOIn_1_LSB_AUX_CONTROL_REG |=  FIFOIn_1_FIFO_CLR;
    FIFOIn_1_LSB_AUX_CONTROL_REG &= ~FIFOIn_1_FIFO_CLR;
    #if (FIFOIn_1_BUS_WIDTH == 16u)
		FIFOIn_1_MSB_AUX_CONTROL_REG |=  FIFOIn_1_FIFO_CLR;
		FIFOIn_1_MSB_AUX_CONTROL_REG &= ~FIFOIn_1_FIFO_CLR;
	#endif	
    /* Exit critical section */
    CyExitCriticalSection(enableInterrupts);
}
