/*******************************************************************************
* File Name: NMIcnt.c  
* Version 3.0
*
*  Description:
*     The Counter component consists of a 8, 16, 24 or 32-bit counter with
*     a selectable period between 2 and 2^Width - 1.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "NMIcnt.h"

uint8 NMIcnt_initVar = 0u;


/*******************************************************************************
* Function Name: NMIcnt_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void NMIcnt_Init(void) 
{
        #if (!NMIcnt_UsingFixedFunction && !NMIcnt_ControlRegRemoved)
            uint8 ctrl;
        #endif /* (!NMIcnt_UsingFixedFunction && !NMIcnt_ControlRegRemoved) */
        
        #if(!NMIcnt_UsingFixedFunction) 
            /* Interrupt State Backup for Critical Region*/
            uint8 NMIcnt_interruptState;
        #endif /* (!NMIcnt_UsingFixedFunction) */
        
        #if (NMIcnt_UsingFixedFunction)
            /* Clear all bits but the enable bit (if it's already set for Timer operation */
            NMIcnt_CONTROL &= NMIcnt_CTRL_ENABLE;
            
            /* Clear the mode bits for continuous run mode */
            #if (CY_PSOC5A)
                NMIcnt_CONTROL2 &= ((uint8)(~NMIcnt_CTRL_MODE_MASK));
            #endif /* (CY_PSOC5A) */
            #if (CY_PSOC3 || CY_PSOC5LP)
                NMIcnt_CONTROL3 &= ((uint8)(~NMIcnt_CTRL_MODE_MASK));                
            #endif /* (CY_PSOC3 || CY_PSOC5LP) */
            /* Check if One Shot mode is enabled i.e. RunMode !=0*/
            #if (NMIcnt_RunModeUsed != 0x0u)
                /* Set 3rd bit of Control register to enable one shot mode */
                NMIcnt_CONTROL |= NMIcnt_ONESHOT;
            #endif /* (NMIcnt_RunModeUsed != 0x0u) */
            
            /* Set the IRQ to use the status register interrupts */
            NMIcnt_CONTROL2 |= NMIcnt_CTRL2_IRQ_SEL;
            
            /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
            NMIcnt_RT1 &= ((uint8)(~NMIcnt_RT1_MASK));
            NMIcnt_RT1 |= NMIcnt_SYNC;     
                    
            /*Enable DSI Sync all all inputs of the Timer*/
            NMIcnt_RT1 &= ((uint8)(~NMIcnt_SYNCDSI_MASK));
            NMIcnt_RT1 |= NMIcnt_SYNCDSI_EN;

        #else
            #if(!NMIcnt_ControlRegRemoved)
            /* Set the default compare mode defined in the parameter */
            ctrl = NMIcnt_CONTROL & ((uint8)(~NMIcnt_CTRL_CMPMODE_MASK));
            NMIcnt_CONTROL = ctrl | NMIcnt_DEFAULT_COMPARE_MODE;
            
            /* Set the default capture mode defined in the parameter */
            ctrl = NMIcnt_CONTROL & ((uint8)(~NMIcnt_CTRL_CAPMODE_MASK));
            
            #if( 0 != NMIcnt_CAPTURE_MODE_CONF)
                NMIcnt_CONTROL = ctrl | NMIcnt_DEFAULT_CAPTURE_MODE;
            #else
                NMIcnt_CONTROL = ctrl;
            #endif /* 0 != NMIcnt_CAPTURE_MODE */ 
            
            #endif /* (!NMIcnt_ControlRegRemoved) */
        #endif /* (NMIcnt_UsingFixedFunction) */
        
        /* Clear all data in the FIFO's */
        #if (!NMIcnt_UsingFixedFunction)
            NMIcnt_ClearFIFO();
        #endif /* (!NMIcnt_UsingFixedFunction) */
        
        /* Set Initial values from Configuration */
        NMIcnt_WritePeriod(NMIcnt_INIT_PERIOD_VALUE);
        #if (!(NMIcnt_UsingFixedFunction && (CY_PSOC5A)))
            NMIcnt_WriteCounter(NMIcnt_INIT_COUNTER_VALUE);
        #endif /* (!(NMIcnt_UsingFixedFunction && (CY_PSOC5A))) */
        NMIcnt_SetInterruptMode(NMIcnt_INIT_INTERRUPTS_MASK);
        
        #if (!NMIcnt_UsingFixedFunction)
            /* Read the status register to clear the unwanted interrupts */
            (void)NMIcnt_ReadStatusRegister();
            /* Set the compare value (only available to non-fixed function implementation */
            NMIcnt_WriteCompare(NMIcnt_INIT_COMPARE_VALUE);
            /* Use the interrupt output of the status register for IRQ output */
            
            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            NMIcnt_interruptState = CyEnterCriticalSection();
            
            NMIcnt_STATUS_AUX_CTRL |= NMIcnt_STATUS_ACTL_INT_EN_MASK;
            
            /* Exit Critical Region*/
            CyExitCriticalSection(NMIcnt_interruptState);
            
        #endif /* (!NMIcnt_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: NMIcnt_Enable
********************************************************************************
* Summary:
*     Enable the Counter
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: 
*   If the Enable mode is set to Hardware only then this function has no effect 
*   on the operation of the counter.
*
*******************************************************************************/
void NMIcnt_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (NMIcnt_UsingFixedFunction)
        NMIcnt_GLOBAL_ENABLE |= NMIcnt_BLOCK_EN_MASK;
        NMIcnt_GLOBAL_STBY_ENABLE |= NMIcnt_BLOCK_STBY_EN_MASK;
    #endif /* (NMIcnt_UsingFixedFunction) */  
        
    /* Enable the counter from the control register  */
    /* If Fixed Function then make sure Mode is set correctly */
    /* else make sure reset is clear */
    #if(!NMIcnt_ControlRegRemoved || NMIcnt_UsingFixedFunction)
        NMIcnt_CONTROL |= NMIcnt_CTRL_ENABLE;                
    #endif /* (!NMIcnt_ControlRegRemoved || NMIcnt_UsingFixedFunction) */
    
}


/*******************************************************************************
* Function Name: NMIcnt_Start
********************************************************************************
* Summary:
*  Enables the counter for operation 
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  NMIcnt_initVar: Is modified when this function is called for the  
*   first time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void NMIcnt_Start(void) 
{
    if(NMIcnt_initVar == 0u)
    {
        NMIcnt_Init();
        
        NMIcnt_initVar = 1u; /* Clear this bit for Initialization */        
    }
    
    /* Enable the Counter */
    NMIcnt_Enable();        
}


/*******************************************************************************
* Function Name: NMIcnt_Stop
********************************************************************************
* Summary:
* Halts the counter, but does not change any modes or disable interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the counter.
*
*******************************************************************************/
void NMIcnt_Stop(void) 
{
    /* Disable Counter */
    #if(!NMIcnt_ControlRegRemoved || NMIcnt_UsingFixedFunction)
        NMIcnt_CONTROL &= ((uint8)(~NMIcnt_CTRL_ENABLE));        
    #endif /* (!NMIcnt_ControlRegRemoved || NMIcnt_UsingFixedFunction) */
    
    /* Globally disable the Fixed Function Block chosen */
    #if (NMIcnt_UsingFixedFunction)
        NMIcnt_GLOBAL_ENABLE &= ((uint8)(~NMIcnt_BLOCK_EN_MASK));
        NMIcnt_GLOBAL_STBY_ENABLE &= ((uint8)(~NMIcnt_BLOCK_STBY_EN_MASK));
    #endif /* (NMIcnt_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: NMIcnt_SetInterruptMode
********************************************************************************
* Summary:
* Configures which interrupt sources are enabled to generate the final interrupt
*
* Parameters:  
*  InterruptsMask: This parameter is an or'd collection of the status bits
*                   which will be allowed to generate the counters interrupt.   
*
* Return: 
*  void
*
*******************************************************************************/
void NMIcnt_SetInterruptMode(uint8 interruptsMask) 
{
    NMIcnt_STATUS_MASK = interruptsMask;
}


/*******************************************************************************
* Function Name: NMIcnt_ReadStatusRegister
********************************************************************************
* Summary:
*   Reads the status register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the status register
*
* Side Effects:
*   Status register bits may be clear on read. 
*
*******************************************************************************/
uint8   NMIcnt_ReadStatusRegister(void) 
{
    return NMIcnt_STATUS;
}


#if(!NMIcnt_ControlRegRemoved)
/*******************************************************************************
* Function Name: NMIcnt_ReadControlRegister
********************************************************************************
* Summary:
*   Reads the control register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
uint8   NMIcnt_ReadControlRegister(void) 
{
    return NMIcnt_CONTROL;
}


/*******************************************************************************
* Function Name: NMIcnt_WriteControlRegister
********************************************************************************
* Summary:
*   Sets the bit-field of the control register.  This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
void    NMIcnt_WriteControlRegister(uint8 control) 
{
    NMIcnt_CONTROL = control;
}

#endif  /* (!NMIcnt_ControlRegRemoved) */


#if (!(NMIcnt_UsingFixedFunction && (CY_PSOC5A)))
/*******************************************************************************
* Function Name: NMIcnt_WriteCounter
********************************************************************************
* Summary:
*   This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
*******************************************************************************/
void NMIcnt_WriteCounter(uint8 counter) \
                                   
{
    #if(NMIcnt_UsingFixedFunction)
        /* assert if block is already enabled */
        CYASSERT (0u == (NMIcnt_GLOBAL_ENABLE & NMIcnt_BLOCK_EN_MASK));
        /* If block is disabled, enable it and then write the counter */
        NMIcnt_GLOBAL_ENABLE |= NMIcnt_BLOCK_EN_MASK;
        CY_SET_REG16(NMIcnt_COUNTER_LSB_PTR, (uint16)counter);
        NMIcnt_GLOBAL_ENABLE &= ((uint8)(~NMIcnt_BLOCK_EN_MASK));
    #else
        CY_SET_REG8(NMIcnt_COUNTER_LSB_PTR, counter);
    #endif /* (NMIcnt_UsingFixedFunction) */
}
#endif /* (!(NMIcnt_UsingFixedFunction && (CY_PSOC5A))) */


/*******************************************************************************
* Function Name: NMIcnt_ReadCounter
********************************************************************************
* Summary:
* Returns the current value of the counter.  It doesn't matter
* if the counter is enabled or running.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint8) The present value of the counter.
*
*******************************************************************************/
uint8 NMIcnt_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    #if(NMIcnt_UsingFixedFunction)
		(void)CY_GET_REG16(NMIcnt_COUNTER_LSB_PTR);
	#else
		(void)CY_GET_REG8(NMIcnt_COUNTER_LSB_PTR_8BIT);
	#endif/* (NMIcnt_UsingFixedFunction) */
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    #if(NMIcnt_UsingFixedFunction)
        return ((uint8)CY_GET_REG16(NMIcnt_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG8(NMIcnt_STATICCOUNT_LSB_PTR));
    #endif /* (NMIcnt_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: NMIcnt_ReadCapture
********************************************************************************
* Summary:
*   This function returns the last value captured.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) Present Capture value.
*
*******************************************************************************/
uint8 NMIcnt_ReadCapture(void) 
{
    #if(NMIcnt_UsingFixedFunction)
        return ((uint8)CY_GET_REG16(NMIcnt_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG8(NMIcnt_STATICCOUNT_LSB_PTR));
    #endif /* (NMIcnt_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: NMIcnt_WritePeriod
********************************************************************************
* Summary:
* Changes the period of the counter.  The new period 
* will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: (uint8) A value of 0 will result in
*         the counter remaining at zero.  
*
* Return: 
*  void
*
*******************************************************************************/
void NMIcnt_WritePeriod(uint8 period) 
{
    #if(NMIcnt_UsingFixedFunction)
        CY_SET_REG16(NMIcnt_PERIOD_LSB_PTR,(uint16)period);
    #else
        CY_SET_REG8(NMIcnt_PERIOD_LSB_PTR, period);
    #endif /* (NMIcnt_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: NMIcnt_ReadPeriod
********************************************************************************
* Summary:
* Reads the current period value without affecting counter operation.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint8) Present period value.
*
*******************************************************************************/
uint8 NMIcnt_ReadPeriod(void) 
{
    #if(NMIcnt_UsingFixedFunction)
        return ((uint8)CY_GET_REG16(NMIcnt_PERIOD_LSB_PTR));
    #else
        return (CY_GET_REG8(NMIcnt_PERIOD_LSB_PTR));
    #endif /* (NMIcnt_UsingFixedFunction) */
}


#if (!NMIcnt_UsingFixedFunction)
/*******************************************************************************
* Function Name: NMIcnt_WriteCompare
********************************************************************************
* Summary:
* Changes the compare value.  The compare output will 
* reflect the new value on the next UDB clock.  The compare output will be 
* driven high when the present counter value compares true based on the 
* configured compare mode setting. 
*
* Parameters:  
*  Compare:  New compare value. 
*
* Return: 
*  void
*
*******************************************************************************/
void NMIcnt_WriteCompare(uint8 compare) \
                                   
{
    #if(NMIcnt_UsingFixedFunction)
        CY_SET_REG16(NMIcnt_COMPARE_LSB_PTR, (uint16)compare);
    #else
        CY_SET_REG8(NMIcnt_COMPARE_LSB_PTR, compare);
    #endif /* (NMIcnt_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: NMIcnt_ReadCompare
********************************************************************************
* Summary:
* Returns the compare value.
*
* Parameters:  
*  void:
*
* Return: 
*  (uint8) Present compare value.
*
*******************************************************************************/
uint8 NMIcnt_ReadCompare(void) 
{
    return (CY_GET_REG8(NMIcnt_COMPARE_LSB_PTR));
}


#if (NMIcnt_COMPARE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: NMIcnt_SetCompareMode
********************************************************************************
* Summary:
*  Sets the software controlled Compare Mode.
*
* Parameters:
*  compareMode:  Compare Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void NMIcnt_SetCompareMode(uint8 compareMode) 
{
    /* Clear the compare mode bits in the control register */
    NMIcnt_CONTROL &= ((uint8)(~NMIcnt_CTRL_CMPMODE_MASK));
    
    /* Write the new setting */
    NMIcnt_CONTROL |= compareMode;
}
#endif  /* (NMIcnt_COMPARE_MODE_SOFTWARE) */


#if (NMIcnt_CAPTURE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: NMIcnt_SetCaptureMode
********************************************************************************
* Summary:
*  Sets the software controlled Capture Mode.
*
* Parameters:
*  captureMode:  Capture Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void NMIcnt_SetCaptureMode(uint8 captureMode) 
{
    /* Clear the capture mode bits in the control register */
    NMIcnt_CONTROL &= ((uint8)(~NMIcnt_CTRL_CAPMODE_MASK));
    
    /* Write the new setting */
    NMIcnt_CONTROL |= ((uint8)((uint8)captureMode << NMIcnt_CTRL_CAPMODE0_SHIFT));
}
#endif  /* (NMIcnt_CAPTURE_MODE_SOFTWARE) */


/*******************************************************************************
* Function Name: NMIcnt_ClearFIFO
********************************************************************************
* Summary:
*   This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void:
*
* Return: 
*  None
*
*******************************************************************************/
void NMIcnt_ClearFIFO(void) 
{

    while(0u != (NMIcnt_ReadStatusRegister() & NMIcnt_STATUS_FIFONEMP))
    {
        (void)NMIcnt_ReadCapture();
    }

}
#endif  /* (!NMIcnt_UsingFixedFunction) */


/* [] END OF FILE */

