/*******************************************************************************
* File Name: NMIcnt_PM.c  
* Version 3.0
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "NMIcnt.h"

static NMIcnt_backupStruct NMIcnt_backup;


/*******************************************************************************
* Function Name: NMIcnt_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  NMIcnt_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void NMIcnt_SaveConfig(void) 
{
    #if (!NMIcnt_UsingFixedFunction)

        NMIcnt_backup.CounterUdb = NMIcnt_ReadCounter();

        #if(!NMIcnt_ControlRegRemoved)
            NMIcnt_backup.CounterControlRegister = NMIcnt_ReadControlRegister();
        #endif /* (!NMIcnt_ControlRegRemoved) */

    #endif /* (!NMIcnt_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: NMIcnt_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  NMIcnt_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void NMIcnt_RestoreConfig(void) 
{      
    #if (!NMIcnt_UsingFixedFunction)

       NMIcnt_WriteCounter(NMIcnt_backup.CounterUdb);

        #if(!NMIcnt_ControlRegRemoved)
            NMIcnt_WriteControlRegister(NMIcnt_backup.CounterControlRegister);
        #endif /* (!NMIcnt_ControlRegRemoved) */

    #endif /* (!NMIcnt_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: NMIcnt_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  NMIcnt_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void NMIcnt_Sleep(void) 
{
    #if(!NMIcnt_ControlRegRemoved)
        /* Save Counter's enable state */
        if(NMIcnt_CTRL_ENABLE == (NMIcnt_CONTROL & NMIcnt_CTRL_ENABLE))
        {
            /* Counter is enabled */
            NMIcnt_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            NMIcnt_backup.CounterEnableState = 0u;
        }
    #else
        NMIcnt_backup.CounterEnableState = 1u;
        if(NMIcnt_backup.CounterEnableState != 0u)
        {
            NMIcnt_backup.CounterEnableState = 0u;
        }
    #endif /* (!NMIcnt_ControlRegRemoved) */
    
    NMIcnt_Stop();
    NMIcnt_SaveConfig();
}


/*******************************************************************************
* Function Name: NMIcnt_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  NMIcnt_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void NMIcnt_Wakeup(void) 
{
    NMIcnt_RestoreConfig();
    #if(!NMIcnt_ControlRegRemoved)
        if(NMIcnt_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            NMIcnt_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!NMIcnt_ControlRegRemoved) */
    
}


/* [] END OF FILE */
